import { BehaviorSubject, Observable, Subject, TeardownLogic } from 'rxjs';
import { IdInterface } from './contract';
import { CustomError } from '@deepkit/core';
export declare class JSONError {
    readonly json: any;
    constructor(json: any);
}
export declare class ValidationErrorItem {
    readonly path: string;
    readonly message: string;
    readonly code: string;
    constructor(path: string, message: string, code: string);
    toString(): string;
}
export declare class ValidationError extends CustomError {
    readonly errors: ValidationErrorItem[];
    constructor(errors: ValidationErrorItem[]);
    static from(errors: {
        path: string;
        message: string;
        code?: string;
    }[]): ValidationError;
    get message(): string;
}
export declare class ValidationParameterError {
    readonly controller: string;
    readonly action: string;
    readonly arg: number;
    readonly errors: ValidationErrorItem[];
    constructor(controller: string, action: string, arg: number, errors: ValidationErrorItem[]);
    get message(): string;
}
export declare function getSerializedErrorPair(error: any): [string, any, any];
export declare function getUnserializedError(entityName: string, error: any, stack: any, info?: string): any;
declare type RegExpForString<T> = T extends string ? (RegExp | T) : T;
declare type MongoAltQuery<T> = T extends Array<infer U> ? (T | RegExpForString<U>) : RegExpForString<T>;
export declare type QuerySelector<T> = {
    $eq?: T;
    $gt?: T;
    $gte?: T;
    $in?: T[];
    $lt?: T;
    $lte?: T;
    $ne?: T;
    $nin?: T[];
    $not?: T extends string ? (QuerySelector<T> | RegExp) : QuerySelector<T>;
    /**
     * When `true`, `$exists` matches the documents that contain the field,
     * including documents where the field value is null.
     */
    $exists?: boolean;
    $type?: any;
    $expr?: any;
    $jsonSchema?: any;
    $mod?: T extends number ? [number, number] : never;
    $regex?: T extends string ? (RegExp | string) : never;
    $options?: T extends string ? string : never;
    $geoIntersects?: {
        $geometry: object;
    };
    $geoWithin?: object;
    $near?: object;
    $nearSphere?: object;
    $maxDistance?: number;
    $all?: T extends Array<infer U> ? any[] : never;
    $elemMatch?: T extends Array<infer U> ? object : never;
    $size?: T extends Array<infer U> ? number : never;
    $bitsAllClear?: any;
    $bitsAllSet?: any;
    $bitsAnyClear?: any;
    $bitsAnySet?: any;
    $parameter?: string;
};
export declare type RootQuerySelector<T> = {
    /** https://docs.mongodb.com/manual/reference/operator/query/and/#op._S_and */
    $and?: Array<FilterQuery<T>>;
    /** https://docs.mongodb.com/manual/reference/operator/query/nor/#op._S_nor */
    $nor?: Array<FilterQuery<T>>;
    /** https://docs.mongodb.com/manual/reference/operator/query/or/#op._S_or */
    $or?: Array<FilterQuery<T>>;
    /** https://docs.mongodb.com/manual/reference/operator/query/text */
    $text?: {
        $search: string;
        $language?: string;
        $caseSensitive?: boolean;
        $diacraticSensitive?: boolean;
    };
    /** https://docs.mongodb.com/manual/reference/operator/query/where/#op._S_where */
    $where?: string | Function;
    /** https://docs.mongodb.com/manual/reference/operator/query/comment/#op._S_comment */
    $comment?: string;
    [key: string]: any;
};
export declare type ObjectQuerySelector<T> = T extends object ? {
    [key in keyof T]?: QuerySelector<T[key]>;
} : QuerySelector<T>;
export declare type Condition<T> = MongoAltQuery<T> | QuerySelector<MongoAltQuery<T>>;
export declare type FilterQuery<T> = {
    [P in keyof T]?: Condition<T[P]>;
} & RootQuerySelector<T>;
export declare class StreamBehaviorSubject<T> extends BehaviorSubject<T> {
    readonly appendSubject: Subject<T>;
    protected nextChange?: Subject<void>;
    protected nextOnAppend: boolean;
    protected unsubscribed: boolean;
    protected teardowns: TeardownLogic[];
    constructor(item: T, teardown?: TeardownLogic);
    isUnsubscribed(): boolean;
    get nextStateChange(): Promise<void>;
    addTearDown(teardown: TeardownLogic): void;
    /**
     * This method differs to BehaviorSubject in the way that this does not throw an error
     * when the subject is closed/unsubscribed.
     */
    getValue(): T;
    next(value: T): void;
    activateNextOnAppend(): void;
    toUTF8(): StreamBehaviorSubject<string>;
    append(value: T): void;
    unsubscribe(): Promise<void>;
}
export declare class EntitySubject<T extends IdInterface> extends StreamBehaviorSubject<T> {
    /**
     * Patches are in class format.
     */
    readonly patches: Subject<{
        [path: string]: any;
    }>;
    readonly delete: Subject<boolean>;
    deleted: boolean;
    get id(): string | number;
    get onDeletion(): Observable<void>;
    next(value: T | undefined): void;
}
export declare type JSONEntity<T> = {
    [P in keyof T]?: any;
};
export {};
