"use strict";
/*
 * Deepkit Framework
 * Copyright (C) 2020 Deepkit UG
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileType = exports.DeepkitFile = exports.FileMode = void 0;
const type_1 = require("@deepkit/type");
const core_1 = require("@deepkit/core");
var FileMode;
(function (FileMode) {
    FileMode[FileMode["closed"] = 0] = "closed";
    FileMode[FileMode["streaming"] = 1] = "streaming";
})(FileMode = exports.FileMode || (exports.FileMode = {}));
let DeepkitFile = class DeepkitFile {
    constructor(
    /**
     * Path WITHOUT starting slash /;
     * e.g.
     *
     *    model.py
     *    .deepkit/log/master.txt
     */
    path) {
        this.path = path;
        this.id = type_1.uuid();
        this.version = 0;
        this.mode = FileMode.closed;
        this.size = 0;
        this.created = new Date();
        this.updated = new Date();
        if (undefined === this.path) {
            throw new Error('new GlutFile undefined path.');
        }
        if (this.path.substr(0, 1) === '/') {
            this.path = this.path.substr(1);
        }
    }
    getMd5() {
        if (!this.md5) {
            throw new Error('File is in streaming mode and has no md5.');
        }
        return this.md5;
    }
    getFullPath() {
        return '/' + this.path;
    }
    getName() {
        const fullPath = '/' + this.path;
        return fullPath.substr(fullPath.lastIndexOf('/') + 1);
    }
    /**
     * Returns always leading slash and trailing slash.
     */
    getDirectory() {
        const fullPath = '/' + this.path;
        return fullPath.substr(0, fullPath.lastIndexOf('/') + 1);
    }
    /**
     * Name without slashes.
     */
    getDirectoryName() {
        const fullPath = '/' + this.path;
        const dirPath = fullPath.substr(0, fullPath.lastIndexOf('/'));
        return dirPath.substr(dirPath.lastIndexOf('/') + 1);
    }
    /**
     * Checks whether this file is in given directory.
     *
     * @param dir with leading slash and trailing slash. Same as getDirectory().
     */
    inDirectory(dir = '/') {
        return this.getDirectory() === dir;
    }
};
__decorate([
    type_1.t.primary.uuid,
    __metadata("design:type", String)
], DeepkitFile.prototype, "id", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], DeepkitFile.prototype, "version", void 0);
__decorate([
    type_1.t.enum(FileMode),
    __metadata("design:type", Number)
], DeepkitFile.prototype, "mode", void 0);
__decorate([
    type_1.t.index(),
    __metadata("design:type", String)
], DeepkitFile.prototype, "md5", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], DeepkitFile.prototype, "size", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Date)
], DeepkitFile.prototype, "created", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Date)
], DeepkitFile.prototype, "updated", void 0);
DeepkitFile = __decorate([
    type_1.Entity('file', 'files'),
    __param(0, type_1.t.name('path').index()),
    __metadata("design:paramtypes", [String])
], DeepkitFile);
exports.DeepkitFile = DeepkitFile;
class FileType {
    constructor(classType) {
        this.classType = classType;
        this.classSchema = type_1.getClassSchema(this.classType);
    }
    static forDefault() {
        return new FileType(DeepkitFile);
    }
    static forCustomType(classType) {
        return new FileType(classType);
    }
    fork(file, newPath) {
        const newFile = new this.classType(newPath);
        for (const i of core_1.eachKey(file)) {
            newFile[i] = file[i];
        }
        newFile.path = newPath;
        newFile.version = 0;
        newFile.created = new Date;
        newFile.updated = new Date;
        newFile.id = type_1.uuid();
        return newFile;
    }
}
exports.FileType = FileType;
//# sourceMappingURL=file.js.map